import json
import os
from pathlib import Path
from typing import List, Dict

def ignored_files_init() -> List[str]:
    """ Initialize a list of ignored files and including README.md, docs.md, etc.
    """
    ignored_files = ["README.md", "tests"]
    for file in os.listdir():
        if file[0] == ".":
            ignored_files.append(file)
    return ignored_files

def read_gitignore() -> List[str]:
    """ Read .gitignore file and return the list of ignored files.
    """
    ignore_files = ignored_files_init()
    try:
        with open(".gitignore", "r") as gitignore:
            for line in gitignore:
                ignore_files.append(line.strip())
    except FileNotFoundError:
        raise ValueError(".gitignore file required for excluding files from documentation generation")
    return ignore_files

def ignore_filepath(filepath: str, ignore_files: List[str]) -> bool:
    """ Check if a filepath should be ignored based on the ignore_files list.
    """
    for part in Path(filepath).parts:
        if part in ignore_files:
            return True
    return False

def get_files() -> Dict:
    """ Retrieve the content of all files in the current directory, excluding those listed in the ignore_files.
    """
    files_dict = {}
    ignore_files = read_gitignore()

    # Iterate over files
    for root, _, files in os.walk("."):
        for file in files:
            filepath = os.path.join(root, file).replace(".\\", "")

            # If not in ignore, collect file text
            if not ignore_filepath(filepath, ignore_files):
                with open(filepath, "r") as f:
                    content = f.read().replace(" " * 4, "\t")
                files_dict[filepath] = content
    return files_dict



import click
import os
from pathlib import Path
import pyperclip
from turbo_docs.utils import directory, openai_api, cli_options


def run_create_readme(text):
    """ Generate a formatted and user-friendly README.md file using the provided text.
    """
    prompt = f"Create a formatted & user-friendly readme.md from the following:\n\n{text}"
    response = openai_api.gpt_completion_wrapper(prompt)
    with open("README.md", "w") as readme_file:
        readme_file.write(response)
    print(f"(--create_readme) Generated README.md")


def run_create_readme_plus(files):
    """ Generate a README.md file with summarized code content from the provided files.
    """
    responses = {}
    for file_path, file_content in files.items():
        if os.stat(file_path).st_size and file_path.split(".")[1]:
            print(f"(--create_readme_plus) Summarizing {file_path}")
            prompt = f"Summarize the following code, especially maintaining key logic:\n{file_content}"
            responses[file_path] = openai_api.gpt_completion_wrapper(prompt)

    psuedocode = "\n\n\n".join([f"{file_path}:\n{summary}" for file_path, summary in responses.items()])
    run_create_readme(psuedocode)
    print(f"(--create_readme_plus) Generated README.md")


def run_create_tests(test_file_path, file_content):
    """ Generate unit tests for the provided code file content and save them in a test file.
    """
    print(f"(--create_tests) Generating unit tests for {test_file_path}")
    prompt = f"Generate unit tests for the following code:\n{file_content}"
    test_content = openai_api.gpt_completion_wrapper(prompt)
    with open(test_file_path, "w") as test_file:
        test_file.write(test_content)


def run_create_tests_helper(files):
    """ Create unit tests for the provided code files and save them in a 'tests' directory.
    """
    tests_dir = "tests"
    Path(tests_dir).mkdir(exist_ok=True)

    for file_path, file_content in files.items():
        if os.stat(file_path).st_size and file_path.split(".")[1]:
            test_file_name = f"test_{os.path.basename(file_path)}"
            test_file_path = os.path.join(tests_dir, test_file_name)
    
            resp = input(f"Would you like to create unit tests for {test_file_name} (Y/n):")
            if resp != "n" and resp != "N":
                run_create_tests(test_file_path, file_content)


@click.command()
@cli_options.copy
@cli_options.create_readme
@cli_options.create_readme_plus
@cli_options.create_tests
def driver(copy: bool, create_readme: bool, create_readme_plus: bool, create_tests: bool) -> None:
    """ The main driver function that handles various command-line options for generating documentation and tests.
    """
    files = directory.get_files()
    dir_text = "\n\n".join([f"{name}:\n\n{content}" for name, content in files.items()])

    # Copy directory text to clipboard if specified
    if copy:
        pyperclip.copy(dir_text)
        print("(--copy) Directory text copied to clipboard")

    # Generate README.md file if specified
    if create_readme:
        run_create_readme(dir_text)

    # Generate docs.md file if specified
    if create_readme_plus:
        run_create_readme_plus(files)

    # Generate unit tests for each code file if specified
    if create_tests:
        run_create_tests_helper(files)


if __name__ == '__main__':
    driver()



import click
from typing import Callable

def copy(func: Callable) -> Callable:
    return click.option(
        '--copy',
        default=False,
        is_flag=True,
        help='Copy the directory text to clipboard'
    )(func)

def create_readme(func: Callable) -> Callable:
    return click.option(
        '--create_readme',
        default=False,
        is_flag=True,
        help='Generate README.md file'
    )(func)

def create_readme_plus(func: Callable) -> Callable:
    return click.option(
        '--create_readme_plus',
        default=False,
        is_flag=True,
        help='Generate readme for larger repositories'
    )(func)

def create_tests(func: Callable) -> Callable:
    return click.option(
        '--create_tests',
        default=False,
        is_flag=True,
        help='Generate unit tests for each code file'
    )(func)



import json
import os
import openai

openai.api_key = os.environ.get('OPENAI_API_KEY')
if not openai.api_key:
    raise ValueError("Cannot find API key. Run the following command: export OPENAI_API_KEY=<your_api_key>")


def gpt_completion_wrapper(prompt):
    """ Send a prompt to the OpenAI GPT-3 API and receive a completion.
    """
    completions = openai.Completion.create(
        engine="text-davinci-003",
        prompt=prompt,
        max_tokens=2048,
        n=1,
        stop=None,
    )
    return completions.choices[0]['text']




from setuptools import setup, find_packages

setup(
    name="turbo_docs",
    version="0.6.0",
    packages=find_packages(),
    install_requires=[
        "requests",
        "openai",
        "click",
        "pyperclip",
    ],
    entry_points={
        "console_scripts": [
            "turbo_docs=turbo_docs.generate:driver"
        ],
    },
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
)
